@@ -52,7 +52,7 @@ class AggregationNode(
         super().__init__(
             Vector,
             [weighted_parent.item for weighted_parent in weighted_parents],
-            persistence_params=PersistenceParams(persist_parent_evaluation_result=True),
+            persistence_params=PersistenceParams(persist_parent_node_result=True),
             dag_effects=dag_effects,
         )
         self._validate_parents()
@@ -94,12 +94,12 @@ class AggregationNode(
         return {
             "weighted_parents": weighted_parents,
             "dag_effects": self.dag_effects,
-            "transformation_config": self.transformation_config,
+            "transformation_config": self.transformation_config._get_embedding_config_parameters(),
         }
 
     @property
     @override
-    def persist_evaluation_result(self) -> bool:
+    def persist_node_result(self) -> bool:
         # Aggregation node's parents are always persisted, no need for double persistence.
         return False
 
@@ -110,6 +110,4 @@ class AggregationNode(
         return []
 
     def project_parents_for_dag_effect(self, dag_effect: DagEffect) -> Sequence[Node]:
-        if dag_effect in self.dag_effects:
-            return self.parents
-        return []
+        return self.parents
