@@ -18,15 +18,11 @@ from superlinked.framework.common.schema.id_schema_object import IdSchemaObject
 
 
 class SchemaObjectReference:
-    def __init__(
-        self, schema: IdSchemaObject, reference_field: SchemaReference
-    ) -> None:
+    def __init__(self, schema: IdSchemaObject, reference_field: SchemaReference) -> None:
         if not isinstance(schema, reference_field._referenced_schema):
-            raise InitializationException(
-                f"{self.__class__.__name__}'s schema is not the referenced schema."
-            )
+            raise InitializationException(f"{self.__class__.__name__}'s schema is not the referenced schema.")
         self.schema = schema
         self.reference_field = reference_field
 
     def __str__(self) -> str:
-        return f"{self.__class__.__name__}(schema={self.schema}, reference_field={self.reference_field})"
+        return f"{self.__class__.__name__}(schema={self.schema._schema_name}, reference_field={self.reference_field})"
