@@ -51,7 +51,6 @@ class LoggerConfigurator:
         expose_pii: bool = False,
         log_as_json: bool = False,
     ) -> None:
-
         if not processors:
             processors = LoggerConfigurator._get_structlog_processors(json_log_file_path, expose_pii, log_as_json)
         log_level = logging.getLogger(PACKAGE_NAME).level
@@ -118,18 +117,18 @@ class LoggerConfigurator:
 
     @staticmethod
     def _get_common_processors(expose_pii: bool = False) -> list[Processor]:
-        shared_processors: list[Processor] = [
+        processors: list[Processor] = [merge_contextvars]
+        if not expose_pii:
+            processors.append(CustomStructlogProcessor.filter_pii)
+        return processors + [
             merge_contextvars,
             CustomStructlogProcessor.evaluate_lazy_arguments,
-            structlog.stdlib.add_logger_name,
             structlog.stdlib.add_log_level,
             structlog.stdlib.PositionalArgumentsFormatter(),
             structlog.stdlib.ExtraAdder(),
+            CustomStructlogProcessor.recursion_handler,
             structlog.processors.TimeStamper(fmt="iso"),
             CustomStructlogProcessor._set_log_var("process_id", os.getpid()),
             structlog.processors.StackInfoRenderer(),
             structlog.processors.UnicodeDecoder(),
         ]
-        if not expose_pii:
-            shared_processors.append(CustomStructlogProcessor.filter_pii)
-        return shared_processors
