@@ -14,6 +14,7 @@
 
 from __future__ import annotations
 
+import numpy as np
 import pandas as pd
 from beartype.typing import Any, Generic, cast
 
@@ -28,15 +29,9 @@ from superlinked.framework.common.parser.parsed_schema import (
     ParsedSchema,
     ParsedSchemaField,
 )
-from superlinked.framework.common.schema.event_schema_object import (
-    EventSchemaObject,
-    SchemaReference,
-)
-from superlinked.framework.common.schema.id_schema_object import (
-    IdSchemaObjectT,
-    SchemaField,
-)
-from superlinked.framework.common.schema.schema_object import Blob, String, Timestamp
+from superlinked.framework.common.schema.event_schema_object import EventSchemaObject
+from superlinked.framework.common.schema.id_schema_object import IdSchemaObjectT
+from superlinked.framework.common.schema.schema_object import Blob, SchemaField
 
 
 class DataFrameParser(Generic[IdSchemaObjectT], DataParser[IdSchemaObjectT, pd.DataFrame]):
@@ -56,20 +51,21 @@ class DataFrameParser(Generic[IdSchemaObjectT], DataParser[IdSchemaObjectT, pd.D
         data_copy = data.copy()
         schema_cols: dict[str, SchemaField] = self._get_column_name_to_schema_field_mapping()
         self._ensure_id(data_copy)
-        if self._is_event_data_parser:
-            self.__ensure_created_at(data_copy)
 
         data_copy[self._id_name] = data_copy[self._id_name].astype(str)
-        self._convert_columns_to_type(data_copy, schema_cols, Timestamp)
-        self._convert_columns_to_type(data_copy, schema_cols, String)
-        self._convert_columns_to_type(data_copy, schema_cols, SchemaReference)
+        self._convert_columns_to_type(data_copy, schema_cols)
 
         if blob_cols := [key for key, value in schema_cols.items() if isinstance(value, Blob)]:
-            data_copy[blob_cols] = data_copy[blob_cols].apply(lambda col: col.map(self.blob_loader.load))
+            for col in blob_cols:
+                data_copy[col] = self.blob_loader.load_multiple(list(data_copy[col]))
 
         if self._is_event_data_parser:
+            self.__ensure_created_at(data_copy)
             data_copy[self._created_at_name] = data_copy[self._created_at_name].astype(int)
-        schema_data = cast(pd.DataFrame, data_copy[list(schema_cols.keys())])
+            self.__ensure_created_at_type(data_copy)
+
+        filtered_cols = {col: schema_field for col, schema_field in schema_cols.items() if col in data_copy.columns}
+        schema_data = cast(pd.DataFrame, data_copy[list(filtered_cols.keys())])
         records = cast(list[dict[str, Any]], schema_data.to_dict(orient="records"))
         return [self.__create_parsed_schema(record, schema_cols) for record in records]
 
@@ -90,9 +86,14 @@ class DataFrameParser(Generic[IdSchemaObjectT], DataParser[IdSchemaObjectT, pd.D
             )
         return ParsedSchema(self._schema, id_, other_fields)
 
+    @staticmethod
+    def _check_value_is_null(value: Any) -> bool:
+        return value is not None and not pd.isnull(value)
+
     def _field_has_non_null_value(self, value: Any) -> bool:
-        values_to_check = value if isinstance(value, list) else [value]
-        return not value or not all(pd.isnull(v) for v in values_to_check)
+        if isinstance(value, (list, np.ndarray)):
+            return len(value) == 0 or any(self._check_value_is_null(v) for v in value)
+        return self._check_value_is_null(value)
 
     def _marshal(
         self,
@@ -146,6 +147,8 @@ class DataFrameParser(Generic[IdSchemaObjectT], DataParser[IdSchemaObjectT, pd.D
                 f"No {self._created_at_name} column in supplied in event dataframe. "
                 f"Create a created_at column with the specified name."
             )
+
+    def __ensure_created_at_type(self, data: pd.DataFrame) -> None:
         if any(
             not self._is_created_at_value_valid(_created_at_val)
             for _created_at_val in data[self._created_at_name].tolist()
@@ -166,8 +169,11 @@ class DataFrameParser(Generic[IdSchemaObjectT], DataParser[IdSchemaObjectT, pd.D
         self,
         data: pd.DataFrame,
         schema_cols: dict[str, SchemaField],
-        schema_field_type: type[SchemaField],
     ) -> None:
         for column_name, schema_field in schema_cols.items():
-            if isinstance(schema_field, schema_field_type):
-                data[column_name] = data[column_name].astype(schema_field.type_)
+            if column_name not in data.columns and schema_field.nullable:
+                continue
+            # pylint: disable=W0640
+            data[column_name] = data[column_name].apply(
+                lambda x: schema_field.as_type(x) if self._field_has_non_null_value(x) else None
+            )
