@@ -13,30 +13,16 @@
 # limitations under the License.
 
 from dataclasses import dataclass
-from enum import Enum
-from pathlib import Path
-
-from typing_extensions import override
 
 from superlinked.framework.common.schema.image_data import ImageData
-from superlinked.framework.common.space.config.embedding.embedding_config import (
-    EmbeddingConfig,
+from superlinked.framework.common.space.config.embedding.model_based_embedding_config import (
+    ModelBasedEmbeddingConfig,
+)
+from superlinked.framework.common.space.embedding.model_based.model_handler import (
+    ModelHandler,
 )
 
 
-class ModelHandler(Enum):
-    SENTENCE_TRANSFORMERS = "sentence_transformers"
-    OPEN_CLIP = "open_clip"
-
-
 @dataclass(frozen=True)
-class ImageEmbeddingConfig(EmbeddingConfig[ImageData]):
-    model_name: str
-    model_cache_dir: Path | None
-    model_handler: ModelHandler
-    length_to_use: int
-
-    @property
-    @override
-    def length(self) -> int:
-        return self.length_to_use
+class ImageEmbeddingConfig(ModelBasedEmbeddingConfig[ImageData, ModelHandler]):
+    pass
