@@ -13,27 +13,19 @@
 # limitations under the License.
 
 from dataclasses import dataclass
-from pathlib import Path
 
-from typing_extensions import override
-
-from superlinked.framework.common.space.config.embedding.embedding_config import (
-    EmbeddingConfig,
+from superlinked.framework.common.space.config.embedding.model_based_embedding_config import (
+    ModelBasedEmbeddingConfig,
+)
+from superlinked.framework.common.space.embedding.model_based.model_handler import (
+    TextModelHandler,
 )
 
 
 @dataclass(frozen=True)
-class TextSimilarityEmbeddingConfig(EmbeddingConfig[str]):
-    model_name: str
-    model_cache_dir: Path | None
-    cache_size: int
-    length_to_use: int
+class TextSimilarityEmbeddingConfig(ModelBasedEmbeddingConfig[str, TextModelHandler]):
+    cache_size: int = 0
 
     def __post_init__(self) -> None:
         if self.cache_size < 0:
             raise ValueError("cache_size must be non-negative")
-
-    @property
-    @override
-    def length(self) -> int:
-        return self.length_to_use
