@@ -21,25 +21,32 @@ from typing_extensions import override
 
 from superlinked.framework.common.const import constants
 from superlinked.framework.common.dag.context import ExecutionContext
-from superlinked.framework.common.data_types import NPArray, Vector
+from superlinked.framework.common.data_types import NPArray, Vector, VectorItemT
 from superlinked.framework.common.space.config.embedding.categorical_similarity_embedding_config import (
     CategoricalSimilarityEmbeddingConfig,
 )
 from superlinked.framework.common.space.embedding.embedding import InvertibleEmbedding
-from superlinked.framework.common.util.math_util import MathUtil
+from superlinked.framework.common.space.embedding.model_based.embedding_engine_manager import (
+    EmbeddingEngineManager,
+)
+from superlinked.framework.common.util.collection_util import CollectionUtil
 
 
 class CategoricalSimilarityEmbedding(InvertibleEmbedding[list[str], CategoricalSimilarityEmbeddingConfig]):
 
-    def __init__(self, embedding_config: CategoricalSimilarityEmbeddingConfig) -> None:
-        super().__init__(embedding_config)
-        self._other_category_name = "".join(sorted(list(self._config.categories)[:100]) + ["_"])
+    def __init__(
+        self, embedding_config: CategoricalSimilarityEmbeddingConfig, embedding_engine_manager: EmbeddingEngineManager
+    ) -> None:
+        super().__init__(embedding_config, embedding_engine_manager)
+        self._other_category_name = (
+            f"{sorted(list(self._config.categories), key=len)[-1] if self._config.categories else ''}_"
+        )
         self._other_category_index: int | None = self.length - 1 if self._config.uncategorized_as_category else None
         self._category_index_map: dict[str, int] = {elem: i for i, elem in enumerate(self._config.categories)}
-        self._default_n_hot_encoding = np.full(self.length, self._config.negative_filter, dtype=np.float64)
+        self._default_n_hot_encoding = np.full(self.length, self._config.negative_filter, dtype=VectorItemT)
 
     @override
-    def embed(self, input_: list[str], context: ExecutionContext) -> Vector:
+    async def embed(self, input_: list[str], context: ExecutionContext) -> Vector:
         n_hot_encoding: NPArray = self._n_hot_encode(input_, context.is_query_context)
         negative_filter_indices = set(i for i in range(self.length) if i not in self._get_category_indices(input_))
         return Vector(n_hot_encoding, negative_filter_indices)
@@ -47,13 +54,9 @@ class CategoricalSimilarityEmbedding(InvertibleEmbedding[list[str], CategoricalS
     @override
     def inverse_embed(self, vector: Vector, context: ExecutionContext) -> list[str]:
         return [
-            (
-                self._config.categories[i]
-                if len(self._config.categories) > i
-                and vector.value[i] != constants.DEFAULT_NOT_AFFECTING_EMBEDDING_VALUE
-                else self._other_category_name
-            )
+            self._config.categories[i] if i < len(self._config.categories) else self._other_category_name
             for i in vector.non_negative_filter_indices
+            if vector.value[i] != constants.DEFAULT_NOT_AFFECTING_EMBEDDING_VALUE
         ]
 
     def get_categorical_encoding_value(self, len_category_list: int, is_query: bool) -> float:
@@ -80,21 +83,26 @@ class CategoricalSimilarityEmbedding(InvertibleEmbedding[list[str], CategoricalS
         )
 
     def _get_index_for_category(self, category: str) -> int | None:
-        return self._category_index_map.get(category, self._other_category_index)
+        return self._category_index_map.get(
+            category, self._other_category_index if self._config.uncategorized_as_category else None
+        )
 
     def _reallocate_vector_values(self, vector: Vector, scaling_factors: Mapping[int, float]) -> Vector:
         if scaling_factors:
-            new_values: NPArray = vector.value
-            new_values[np.array(list(scaling_factors.keys()))] *= np.array(list(scaling_factors.values()))
-            sum_values = MathUtil.get_max_signed_sum(new_values.tolist())
+            new_values = np.copy(vector.value)
+            indices = np.array(list(scaling_factors.keys()), dtype=np.int64)
+            factors = np.array(list(scaling_factors.values()), dtype=np.float64)
+            new_values[indices] = new_values[indices] * factors
+            new_vector = Vector(new_values, vector.negative_filter_indices)
+            sum_values = sum(CollectionUtil.get_positive_values_ndarray(new_vector.value))
             normalizing_factor = sum_values / math.sqrt(len(self._config.categories))
-            return Vector(new_values, vector.negative_filter_indices).normalize(normalizing_factor)
+            return new_vector.normalize(normalizing_factor)
         return vector
 
     @override
-    def _to_query_vector(self, input_: Vector, context: ExecutionContext) -> Vector:
+    async def _to_query_vector(self, input_: Vector, context: ExecutionContext) -> Vector:
         category_vector_values: dict[int, float] = self._get_scaling_factors_for_vector(input_)
-        raw_vector: Vector = self.embed(self.inverse_embed(input_, context), context)
+        raw_vector: Vector = await self.embed(self.inverse_embed(input_, context), context)
         reallocated_vector: Vector = self._reallocate_vector_values(raw_vector, category_vector_values)
         return reallocated_vector
 
