@@ -15,3 +15,15 @@
 
 class EncoderException(Exception):
     pass
+
+
+class InvalidIndexConfigException(Exception):
+    pass
+
+
+class IndexConfigNotFoundException(Exception):
+    pass
+
+
+class MismatchingConfigException(Exception):
+    pass
