@@ -23,5 +23,7 @@ class FieldDataType(Enum):
     INT = "INT"
     JSON = "JSON"
     STRING = "STRING"
+    BOOLEAN = "BOOLEAN"
+    METADATA_STRING = "METADATA_STRING"  # so admin fields can be Tags in redis
     STRING_LIST = "STRING_LIST"
     VECTOR = "VECTOR"
