@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 from dataclasses import dataclass
 
 from beartype.typing import Sequence
@@ -21,6 +23,7 @@ from superlinked.framework.common.interface.comparison_operand import (
     ComparisonOperation,
 )
 from superlinked.framework.common.schema.schema_object import SchemaField
+from superlinked.framework.dsl.query.clause_params import KNNSearchClauseParams
 
 
 @dataclass(frozen=True)
@@ -29,4 +32,18 @@ class KNNSearchParams:
     limit: int
     filters: Sequence[ComparisonOperation[SchemaField]]
     schema_fields_to_return: Sequence[SchemaField]
-    radius: float | None = None
+    radius: float | None
+    should_return_index_vector: bool = False
+
+    @classmethod
+    def from_clause_params(cls, query_vector: Vector, partial: KNNSearchClauseParams) -> KNNSearchParams:
+        if partial.limit is None:
+            raise ValueError(f"{cls.__name__} must have a valid limit, got None.")
+        return cls(
+            query_vector,
+            partial.limit,
+            partial.filters,
+            partial.schema_fields_to_return,
+            partial.radius,
+            partial.should_return_index_vector,
+        )
