@@ -16,6 +16,7 @@
 from beartype.typing import Any, Sequence
 
 from superlinked.framework.common.exception import QueryException
+from superlinked.framework.common.telemetry.telemetry_registry import telemetry
 from superlinked.framework.common.util.async_util import AsyncUtil
 from superlinked.framework.common.util.type_validator import TypeValidator
 from superlinked.framework.dsl.executor.query.query_executor import QueryExecutor
@@ -70,7 +71,11 @@ class QueryMixin:
         Raises:
             QueryException: If the query index is not found among the executor's indices.
         """
-        return AsyncUtil.run(self.async_query(query_descriptor, **params))
+        with telemetry.span(
+            "executor.query",
+            attributes={"index_id": query_descriptor.index._node_id, "schema": query_descriptor.schema._schema_name},
+        ):
+            return AsyncUtil.run(self.async_query(query_descriptor, **params))
 
     async def async_query(self, query_descriptor: QueryDescriptor, **params: Any) -> QueryResult:
         if query_vector_factory := self._query_vector_factory_by_index.get(query_descriptor.index):
